#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" State variables for analysis solutions such as powerflow.State variables for analysis solutions such as powerflow.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from UCTE import Element
from UCTE.Domain import AngleRadians
from UCTE.Domain import Voltage
from UCTE.Domain import ActivePower
from UCTE.Domain import ReactivePower



from enthought.traits.api import Instance, List, Property, Float
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#  "StateVariable" class:
#------------------------------------------------------------------------------

class StateVariable(Element):
    """ An abstract class for state variables.An abstract class for state variables.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "StateVariable" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID",
                label="Attributes"),
            VGroup("Model",
                label="References"),
            dock="tab"),
        id="UCTE.StateVariables.StateVariable",
        title="StateVariable",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "StateVariable" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SvVoltage" class:
#------------------------------------------------------------------------------

class SvVoltage(StateVariable):
    """ State variable for voltage.State variable for voltage.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The topological node associated with the voltage state.The topological node associated with the voltage state.
    TopologicalNode = Instance("UCTE.Topology.TopologicalNode", allow_none=False,
        desc="The topological node associated with the voltage state.The topological node associated with the voltage state.",
        transient=True,
        opposite="SvVoltage",
        editor=InstanceEditor(name="_topologicalnodes"))

    def _get_topologicalnodes(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Topology.TopologicalNode" ]
        else:
            return []

    _topologicalnodes = Property(fget=_get_topologicalnodes)

    # The voltage angle in radians of the topological node.The voltage angle in radians of the topological node.
    angle = AngleRadians(desc="The voltage angle in radians of the topological node.The voltage angle in radians of the topological node.")

    # The voltage magnitude of the topological node.The voltage magnitude of the topological node.
    v = Voltage(desc="The voltage magnitude of the topological node.The voltage magnitude of the topological node.")

    #--------------------------------------------------------------------------
    #  Begin "SvVoltage" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "angle", "v",
                label="Attributes"),
            VGroup("Model", "TopologicalNode",
                label="References"),
            dock="tab"),
        id="UCTE.StateVariables.SvVoltage",
        title="SvVoltage",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SvVoltage" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SvShuntCompensatorSections" class:
#------------------------------------------------------------------------------

class SvShuntCompensatorSections(StateVariable):
    """ State variable for the number of sections in service for a shunt compensator.A SvShuntCompensator is always associated with any instance of ShuntCompensator.   The sections or continuousSections values are specified depending upon the value of the associated RegulatingControl.discrete attribute.  If no RegulatingControl is associated, then the ShuntCompensator is treated as discrete.    In discrete mode, the 'sections' attribute must be present.   In the not 'discrete' mode (continuous mode) the 'continuousSections' attribute must be present.     In the case the Terminal.connected value is 'false' the specificed number of sections is not meaningful to the powerflow solution and powerflow implementations should interpret this as zero injection.   Note that an SvShuntCompensatorSections should be supplied even for ShuntCompensators whose Terminal.connected status is 'false' to keep total number of ShuntCompensator and SvShuntCompensatorSection objects in the model the same.State variable for the number of sections in service for a shunt compensator.A SvShuntCompensator is always associated with any instance of ShuntCompensator.   The sections or continuousSections values are specified depending upon the value of the associated RegulatingControl.discrete attribute.  If no RegulatingControl is associated, then the ShuntCompensator is treated as discrete.    In discrete mode, the 'sections' attribute must be present.   In the not 'discrete' mode (continuous mode) the 'continuousSections' attribute must be present.     In the case the Terminal.connected value is 'false' the specificed number of sections is not meaningful to the powerflow solution and powerflow implementations should interpret this as zero injection.   Note that an SvShuntCompensatorSections should be supplied even for ShuntCompensators whose Terminal.connected status is 'false' to keep total number of ShuntCompensator and SvShuntCompensatorSection objects in the model the same.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The shunt compensator for which the state applies.The shunt compensator for which the state applies.
    ShuntCompensator = Instance("UCTE.Wires.ShuntCompensator", allow_none=False,
        desc="The shunt compensator for which the state applies.The shunt compensator for which the state applies.",
        transient=True,
        opposite="SvShuntCompensatorSections",
        editor=InstanceEditor(name="_shuntcompensators"))

    def _get_shuntcompensators(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Wires.ShuntCompensator" ]
        else:
            return []

    _shuntcompensators = Property(fget=_get_shuntcompensators)

    # The number of sections in service as a continous variable.The number of sections in service as a continous variable.
    continuousSections = Float(desc="The number of sections in service as a continous variable.The number of sections in service as a continous variable.")

    #--------------------------------------------------------------------------
    #  Begin "SvShuntCompensatorSections" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "continuousSections",
                label="Attributes"),
            VGroup("Model", "ShuntCompensator",
                label="References"),
            dock="tab"),
        id="UCTE.StateVariables.SvShuntCompensatorSections",
        title="SvShuntCompensatorSections",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SvShuntCompensatorSections" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SvTapStep" class:
#------------------------------------------------------------------------------

class SvTapStep(StateVariable):
    """ State variable for transformer tap step.     Normally a profile specifies only one of the attributes 'position'or 'continuousPosition'.SvTapStep is only meant to be used for taps that change under load.State variable for transformer tap step.     Normally a profile specifies only one of the attributes 'position'or 'continuousPosition'.SvTapStep is only meant to be used for taps that change under load.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The tap changer associated with the tap step state.The tap changer associated with the tap step state.
    TapChanger = Instance("UCTE.Wires.TapChanger", allow_none=False,
        desc="The tap changer associated with the tap step state.The tap changer associated with the tap step state.",
        transient=True,
        opposite="SvTapStep",
        editor=InstanceEditor(name="_tapchangers"))

    def _get_tapchangers(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Wires.TapChanger" ]
        else:
            return []

    _tapchangers = Property(fget=_get_tapchangers)

    # The floating point tap position.The floating point tap position.
    continuousPosition = Float(desc="The floating point tap position.The floating point tap position.")

    #--------------------------------------------------------------------------
    #  Begin "SvTapStep" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "continuousPosition",
                label="Attributes"),
            VGroup("Model", "TapChanger",
                label="References"),
            dock="tab"),
        id="UCTE.StateVariables.SvTapStep",
        title="SvTapStep",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SvTapStep" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SvPowerFlow" class:
#------------------------------------------------------------------------------

class SvPowerFlow(StateVariable):
    """ State variable for power flow.Only Terminal instances of EnergyConsumer and SynchronousMachine will have SvPowerFlow instances assigned.   The number of SvPowerFlow instances in the model should match the number EnergyConsumer plus SynchronousMachine objects in the model regardless of the Terminal.connected values.    Any SvPowerFlow with a Terminal.connected value of false must have zero flow explicitly specified on an SvPowerFlow instance. The other types of terminals are not included in exchanges since their values can be readily computed from local voltages and attributes without a global powerflow solution.State variable for power flow.Only Terminal instances of EnergyConsumer and SynchronousMachine will have SvPowerFlow instances assigned.   The number of SvPowerFlow instances in the model should match the number EnergyConsumer plus SynchronousMachine objects in the model regardless of the Terminal.connected values.    Any SvPowerFlow with a Terminal.connected value of false must have zero flow explicitly specified on an SvPowerFlow instance. The other types of terminals are not included in exchanges since their values can be readily computed from local voltages and attributes without a global powerflow solution.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The terminal associated with the power flow state.The SvPowerFlow is only associated with the Terminal objects of shunt injection classes EnergyConsumer and  SynchronousMachine.  Branch flows are not exchanged since they can be readily computed from the voltages, impedances, and for transformers additionally the tap parameters including the SvTapStep.  Similarly, Switch flows are not included because they can typically be uniquely computed, except in the case of meshed networks of Switch objects.  Terminals of the ShuntCompensator class are not associated because the injection value can be computed from the solved voltage, number of sections, Termianl.connected state, and the impedance per section attributes on the ShuntCompensator. The terminal associated with the power flow state.The SvPowerFlow is only associated with the Terminal objects of shunt injection classes EnergyConsumer and  SynchronousMachine.  Branch flows are not exchanged since they can be readily computed from the voltages, impedances, and for transformers additionally the tap parameters including the SvTapStep.  Similarly, Switch flows are not included because they can typically be uniquely computed, except in the case of meshed networks of Switch objects.  Terminals of the ShuntCompensator class are not associated because the injection value can be computed from the solved voltage, number of sections, Termianl.connected state, and the impedance per section attributes on the ShuntCompensator. 
    Terminal = Instance("UCTE.Core.Terminal", allow_none=False,
        desc="The terminal associated with the power flow state.The SvPowerFlow is only associated with the Terminal objects of shunt injection classes EnergyConsumer and  SynchronousMachine.  Branch flows are not exchanged since they can be readily computed from the voltages, impedances, and for transformers additionally the tap parameters including the SvTapStep.  Similarly, Switch flows are not included because they can typically be uniquely computed, except in the case of meshed networks of Switch objects.  Terminals of the ShuntCompensator class are not associated because the injection value can be computed from the solved voltage, number of sections, Termianl.connected state, and the impedance per section attributes on the ShuntCompensator. The terminal associated with the power flow state.The SvPowerFlow is only associated with the Terminal objects of shunt injection classes EnergyConsumer and  SynchronousMachine.  Branch flows are not exchanged since they can be readily computed from the voltages, impedances, and for transformers additionally the tap parameters including the SvTapStep.  Similarly, Switch flows are not included because they can typically be uniquely computed, except in the case of meshed networks of Switch objects.  Terminals of the ShuntCompensator class are not associated because the injection value can be computed from the solved voltage, number of sections, Termianl.connected state, and the impedance per section attributes on the ShuntCompensator. ",
        transient=True,
        opposite="SvPowerFlow",
        editor=InstanceEditor(name="_terminals"))

    def _get_terminals(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "UCTE.Core.Terminal" ]
        else:
            return []

    _terminals = Property(fget=_get_terminals)

    # The active power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.p should be zero.   The power flow is into the Terminal of the ConductingEquipment.The active power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.p should be zero.   The power flow is into the Terminal of the ConductingEquipment.
    p = ActivePower(desc="The active power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.p should be zero.   The power flow is into the Terminal of the ConductingEquipment.The active power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.p should be zero.   The power flow is into the Terminal of the ConductingEquipment.")

    # The reactive power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.q should be zero.   The power flow is into the Terminal of the ConductingEquipment.The reactive power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.q should be zero.   The power flow is into the Terminal of the ConductingEquipment.
    q = ReactivePower(desc="The reactive power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.q should be zero.   The power flow is into the Terminal of the ConductingEquipment.The reactive power flow into the terminal.If the associated Terminal.connected status is 'false', the flow specified in the SvPowerFlow.q should be zero.   The power flow is into the Terminal of the ConductingEquipment.")

    #--------------------------------------------------------------------------
    #  Begin "SvPowerFlow" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "p", "q",
                label="Attributes"),
            VGroup("Model", "Terminal",
                label="References"),
            dock="tab"),
        id="UCTE.StateVariables.SvPowerFlow",
        title="SvPowerFlow",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SvPowerFlow" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" State variables for analysis solutions such as powerflow.State variables for analysis solutions such as powerflow.
"""

from cdpsm import Element

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim"

ns_uri = "http://iec.ch/TC57/2009/CIM-schema-cim14#Package_StateVariables"

class SvTapStep(Element):
    """ State variable for transformer tap step.     This class is to be used for taps of LTC (load tap changing) transformers, not fixed tap transformers.  Normally a profile specifies only one of the attributes 'position'or 'tapRatio'.State variable for transformer tap step.     This class is to be used for taps of LTC (load tap changing) transformers, not fixed tap transformers.  Normally a profile specifies only one of the attributes 'position'or 'tapRatio'.
    """
    # <<< sv_tap_step
    # @generated
    def __init__(self, continuous_position=0.0, position=0, tap_changer=None, **kw_args):
        """ Initialises a new 'SvTapStep' instance.
        """
        # The floating point tap position.The floating point tap position. 
        self.continuous_position = continuous_position

        # The integer tap position.The integer tap position. 
        self.position = position


        self._tap_changer = None
        self.tap_changer = tap_changer


        super(SvTapStep, self).__init__(**kw_args)
    # >>> sv_tap_step

    # <<< tap_changer
    # @generated
    def get_tap_changer(self):
        """ The tap changer associated with the tap step state.The tap changer associated with the tap step state.
        """
        return self._tap_changer

    def set_tap_changer(self, value):
        if self._tap_changer is not None:
            self._tap_changer._sv_tap_step = None

        self._tap_changer = value
        if self._tap_changer is not None:
            self._tap_changer._sv_tap_step = self

    tap_changer = property(get_tap_changer, set_tap_changer)
    # >>> tap_changer


    def __str__(self):
        """ Returns a string representation of the SvTapStep.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< sv_tap_step.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the SvTapStep.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "SvTapStep", self.uri)
        if format:
            indent += ' ' * depth

        if self.tap_changer is not None:
            s += '%s<%s:SvTapStep.tap_changer rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.tap_changer.uri)
        s += '%s<%s:SvTapStep.continuous_position>%s</%s:SvTapStep.continuous_position>' % \
            (indent, ns_prefix, self.continuous_position, ns_prefix)
        s += '%s<%s:SvTapStep.position>%s</%s:SvTapStep.position>' % \
            (indent, ns_prefix, self.position, ns_prefix)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "SvTapStep")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> sv_tap_step.serialize


# <<< state_variables
# @generated
# >>> state_variables



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61968.Common import Document
from CIM.IEC61970.Core import IdentifiedObject
from CIM.IEC61970.Domain import StringQuantity
from CIM.IEC61970.Domain import Money
from CIM.IEC61970.Domain import ApparentPower
from CIM.IEC61970.Core import PhaseCode
from CIM.IEC61970.Domain import Voltage
from CIM.IEC61970.Domain import ActivePower
from CIM.IEC61970.Domain import Length
from CIM.IEC61970.Domain import CurrentFlow
from CIM.IEC61970.Domain import Resistance
from CIM.IEC61970.Domain import Reactance
from CIM.IEC61970.Domain import ReactivePower



from enthought.traits.api import Instance, List, Property, Enum, Bool, Int, Float, Str
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------

# Kind of resetting the fault indicators.
FaultIndicatorResetKind = Enum("other", "remote", "manual", "automatic", desc="Kind of resetting the fault indicators.")

#------------------------------------------------------------------------------
#  "TypeAsset" class:
#------------------------------------------------------------------------------

class TypeAsset(Document):
    """ Whereas an AssetModel is a particular model and version of a vendor's product, a TypeAsset is documentation for a generic asset or material item that may be used for design purposes. Any number of AssetModels may be used to perform this generic function. The primary role of the TypeAsset is typically defined by the PowereSystemResource it is associated with.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    CUWorkEquipmentAsset = Instance("CIM.IEC61968.Informative.InfWork.CUWorkEquipmentItem",
        transient=True,
        opposite="TypeAsset",
        editor=InstanceEditor(name="_cuworkequipmentitems"))

    def _get_cuworkequipmentitems(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfWork.CUWorkEquipmentItem" ]
        else:
            return []

    _cuworkequipmentitems = Property(fget=_get_cuworkequipmentitems)

    CUAsset = Instance("CIM.IEC61968.Informative.InfWork.CUAsset",
        transient=True,
        opposite="TypeAsset",
        editor=InstanceEditor(name="_cuassets"))

    def _get_cuassets(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfWork.CUAsset" ]
        else:
            return []

    _cuassets = Property(fget=_get_cuassets)

    TypeAssetCatalogue = Instance("CIM.IEC61968.Informative.InfTypeAsset.TypeAssetCatalogue",
        transient=True,
        opposite="TypeAssets",
        editor=InstanceEditor(name="_typeassetcatalogues"))

    def _get_typeassetcatalogues(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfTypeAsset.TypeAssetCatalogue" ]
        else:
            return []

    _typeassetcatalogues = Property(fget=_get_typeassetcatalogues)

    # A type of asset may be satisified with many different types of asset models.
    AssetModels = List(Instance("CIM.IEC61968.AssetModels.AssetModel"),
        desc="A type of asset may be satisified with many different types of asset models.")

    ErpInventoryIssues = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpIssueInventory"))

    ErpReqLineItems = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpReqLineItem"))

    ErpBomItemDatas = List(Instance("CIM.IEC61968.Informative.InfERPSupport.ErpBomItemData"))

    # The value, unit of measure, and multiplier for the quantity.
    quantity = StringQuantity(desc="The value, unit of measure, and multiplier for the quantity.")

    # True if item is a stock item (default).
    stockItem = Bool(desc="True if item is a stock item (default).")

    # Estimated unit cost (or cost per unit length) of this type of asset. It does not include labor to install/construct or configure it.
    estimatedUnitCost = Money(desc="Estimated unit cost (or cost per unit length) of this type of asset. It does not include labor to install/construct or configure it.")

    #--------------------------------------------------------------------------
    #  Begin "TypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.TypeAsset",
        title="TypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "MountingPoint" class:
#------------------------------------------------------------------------------

class MountingPoint(IdentifiedObject):
    """ Point on a structure that a connection may have a conductor connected to. Defined with an x and y coordinate plus a phase. A connection may have multiple mounting points, one for each phase.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    OverheadConductors = List(Instance("CIM.IEC61968.Informative.InfAssets.OverheadConductorAsset"))

    Connections = List(Instance("CIM.IEC61968.Informative.InfTypeAsset.Connection"))

    phaseCode = PhaseCode

    yCoord = Int

    xCoord = Int

    #--------------------------------------------------------------------------
    #  Begin "MountingPoint" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "phaseCode", "yCoord", "xCoord",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OverheadConductors", "Connections",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.MountingPoint",
        title="MountingPoint",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "MountingPoint" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TypeAssetCatalogue" class:
#------------------------------------------------------------------------------

class TypeAssetCatalogue(IdentifiedObject):
    """ Catalogue of generic types of assets (TypeAsset) that may be used for design purposes. It is not associated with a particular manufacturer.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    TypeAssets = List(Instance("CIM.IEC61968.Informative.InfTypeAsset.TypeAsset"))

    status = Instance("CIM.IEC61968.Common.Status",
        transient=True,
        editor=InstanceEditor(name="_statuss"))

    def _get_statuss(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Common.Status" ]
        else:
            return []

    _statuss = Property(fget=_get_statuss)

    #--------------------------------------------------------------------------
    #  Begin "TypeAssetCatalogue" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "TypeAssets", "status",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.TypeAssetCatalogue",
        title="TypeAssetCatalogue",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TypeAssetCatalogue" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "Connection" class:
#------------------------------------------------------------------------------

class Connection(IdentifiedObject):
    """ A structure can have multiple connection points for electrical connections (e.g. line) each with multiple mounting points, one for each phase. e.g. a Tower may have three Connections, two with three mounting points, one for each phase and a third with a single mounting point for the neutral line. A pole, on the other hand, may have a single Connection with one, two or three mounting points depending on whether it is carrying 1,2 or 3 phases.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    StructureTypeAssets = List(Instance("CIM.IEC61968.Informative.InfTypeAsset.StructureTypeAsset"))

    MountingPoints = List(Instance("CIM.IEC61968.Informative.InfTypeAsset.MountingPoint"))

    #--------------------------------------------------------------------------
    #  Begin "Connection" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "StructureTypeAssets", "MountingPoints",
                label="References"),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.Connection",
        title="Connection",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "Connection" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ElectricalTypeAsset" class:
#------------------------------------------------------------------------------

class ElectricalTypeAsset(TypeAsset):
    """ Generic TypeAsset for all types of component in the network that have electrical characteristics.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ElectricalInfos = List(Instance("CIM.IEC61968.Assets.ElectricalInfo"))

    #--------------------------------------------------------------------------
    #  Begin "ElectricalTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.ElectricalTypeAsset",
        title="ElectricalTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ElectricalTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ShuntCompensatorTypeAsset" class:
#------------------------------------------------------------------------------

class ShuntCompensatorTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic shunt compensator that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ShuntImpedanceInfo = Instance("CIM.IEC61968.Informative.InfAssets.ShuntImpedanceInfo",
        transient=True,
        opposite="ShuntCompensatorTypeAsset",
        editor=InstanceEditor(name="_shuntimpedanceinfos"))

    def _get_shuntimpedanceinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.ShuntImpedanceInfo" ]
        else:
            return []

    _shuntimpedanceinfos = Property(fget=_get_shuntimpedanceinfos)

    ShuntCompensatorAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.ShuntCompensatorAssetModel"))

    # Maximum allowed Apparent Power loss
    maxPowerLoss = ApparentPower(desc="Maximum allowed Apparent Power loss")

    #--------------------------------------------------------------------------
    #  Begin "ShuntCompensatorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "maxPowerLoss",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "ShuntImpedanceInfo", "ShuntCompensatorAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.ShuntCompensatorTypeAsset",
        title="ShuntCompensatorTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ShuntCompensatorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EndDeviceTypeAsset" class:
#------------------------------------------------------------------------------

class EndDeviceTypeAsset(TypeAsset):
    """ Documentation for generic End Device that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    EndDeviceModels = List(Instance("CIM.IEC61968.AssetModels.EndDeviceModel"))

    #--------------------------------------------------------------------------
    #  Begin "EndDeviceTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "EndDeviceModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.EndDeviceTypeAsset",
        title="EndDeviceTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EndDeviceTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "BusbarTypeAsset" class:
#------------------------------------------------------------------------------

class BusbarTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic busbar that may be used for design purposes. It is typically associated with PoserSystemResource BusbarSection.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    BusbarTypeAssets = List(Instance("CIM.IEC61968.Informative.InfAssetModels.BusbarAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "BusbarTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "BusbarTypeAssets",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.BusbarTypeAsset",
        title="BusbarTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "BusbarTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "WorkEquipmentTypeAsset" class:
#------------------------------------------------------------------------------

class WorkEquipmentTypeAsset(TypeAsset):
    """ Documentation for generic equipment that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    WorkEquipmentAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.WorkEquipmentAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "WorkEquipmentTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "WorkEquipmentAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.WorkEquipmentTypeAsset",
        title="WorkEquipmentTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "WorkEquipmentTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SwitchTypeAsset" class:
#------------------------------------------------------------------------------

class SwitchTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic switch asset that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    SwitchAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.SwitchAssetModel"))

    CompositeSwitchTypeAsset = Instance("CIM.IEC61968.Informative.InfTypeAsset.CompositeSwitchTypeAsset",
        transient=True,
        opposite="SwitchTypesAssets",
        editor=InstanceEditor(name="_compositeswitchtypeassets"))

    def _get_compositeswitchtypeassets(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfTypeAsset.CompositeSwitchTypeAsset" ]
        else:
            return []

    _compositeswitchtypeassets = Property(fget=_get_compositeswitchtypeassets)

    SwitchInfo = Instance("CIM.IEC61968.Informative.InfAssets.SwitchInfo",
        transient=True,
        opposite="SwitchTypeAsset",
        editor=InstanceEditor(name="_switchinfos"))

    def _get_switchinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.SwitchInfo" ]
        else:
            return []

    _switchinfos = Property(fget=_get_switchinfos)

    #--------------------------------------------------------------------------
    #  Begin "SwitchTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "SwitchAssetModels", "CompositeSwitchTypeAsset", "SwitchInfo",
                label="References", columns=3),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.SwitchTypeAsset",
        title="SwitchTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SwitchTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "StructureTypeAsset" class:
#------------------------------------------------------------------------------

class StructureTypeAsset(TypeAsset):
    """ A Type of Structural Asset with properties common to a large number of asset models.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    MountConnections = List(Instance("CIM.IEC61968.Informative.InfTypeAsset.Connection"))

    # Maximum rated voltage of the equipment that can be mounted on/contained within the structure.
    ratedVoltage = Voltage(desc="Maximum rated voltage of the equipment that can be mounted on/contained within the structure.")

    #--------------------------------------------------------------------------
    #  Begin "StructureTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "ratedVoltage",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "MountConnections",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.StructureTypeAsset",
        title="StructureTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "StructureTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "TowerTypeAsset" class:
#------------------------------------------------------------------------------

class TowerTypeAsset(StructureTypeAsset):
    """ Documentation for a generic tower that may be used for various purposes such as work planning. A transmission tower carrying two 3-phase circuits will have 2 instances of Connection, each of which will have 3 MountingPoint instances, one for each phase all with coordinates relative to a common origin on the tower. (It may also have a 3rd Connection with a single MountingPoint for the Neutral line).
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    TowerAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.TowerAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "TowerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "ratedVoltage",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "MountConnections", "TowerAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.TowerTypeAsset",
        title="TowerTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "TowerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "StreetlightTypeAsset" class:
#------------------------------------------------------------------------------

class StreetlightTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic streetlight that may be used for various purposes such as work planning. Use 'category' for utility specific categorisation, such as luminar, grid light, lantern, open bottom, flood, etc.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    StreetlightAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.StreetlightAssetModel"))

    # Nominal (as designed) power rating of light.
    lightRating = ActivePower(desc="Nominal (as designed) power rating of light.")

    #--------------------------------------------------------------------------
    #  Begin "StreetlightTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "lightRating",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "StreetlightAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.StreetlightTypeAsset",
        title="StreetlightTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "StreetlightTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ToolTypeAsset" class:
#------------------------------------------------------------------------------

class ToolTypeAsset(TypeAsset):
    """ Documentation for a generic tool that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ToolAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.ToolAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "ToolTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ToolAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.ToolTypeAsset",
        title="ToolTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ToolTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CompositeSwitchTypeAsset" class:
#------------------------------------------------------------------------------

class CompositeSwitchTypeAsset(TypeAsset):
    """ Documentation for a generic composite switch asset that may be used for design purposes. A composite wwitch is an amalgamation of multiple Switches.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    CompositeSwitchAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.CompositeSwitchAssetModel"))

    SwitchTypesAssets = List(Instance("CIM.IEC61968.Informative.InfTypeAsset.SwitchTypeAsset"))

    CompositeSwitchInfo = Instance("CIM.IEC61968.Informative.InfAssets.CompositeSwitchInfo",
        transient=True,
        opposite="CompositeSwitchTypeAsset",
        editor=InstanceEditor(name="_compositeswitchinfos"))

    def _get_compositeswitchinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.CompositeSwitchInfo" ]
        else:
            return []

    _compositeswitchinfos = Property(fget=_get_compositeswitchinfos)

    #--------------------------------------------------------------------------
    #  Begin "CompositeSwitchTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "CompositeSwitchAssetModels", "SwitchTypesAssets", "CompositeSwitchInfo",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.CompositeSwitchTypeAsset",
        title="CompositeSwitchTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CompositeSwitchTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ComEquipmentTypeAsset" class:
#------------------------------------------------------------------------------

class ComEquipmentTypeAsset(TypeAsset):
    """ Documentation for a piece of Communication Equipment (e.g., gateway, router, network hub, etc.) that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "ComEquipmentTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.ComEquipmentTypeAsset",
        title="ComEquipmentTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ComEquipmentTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "FACTSDeviceTypeAsset" class:
#------------------------------------------------------------------------------

class FACTSDeviceTypeAsset(ElectricalTypeAsset):
    """ Documentation for generic Flexible alternating current transmission systems (FACTS) devices that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    FACTSDeviceAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.FACTSDeviceAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "FACTSDeviceTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "FACTSDeviceAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.FACTSDeviceTypeAsset",
        title="FACTSDeviceTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "FACTSDeviceTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SVCTypeAsset" class:
#------------------------------------------------------------------------------

class SVCTypeAsset(FACTSDeviceTypeAsset):
    """ Documentation for a generic Static Var Compensator (SVC) that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    SvcInfos = List(Instance("CIM.IEC61968.Informative.InfAssets.SVCInfo"))

    SVCAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.SVCAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "SVCTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "FACTSDeviceAssetModels", "SvcInfos", "SVCAssetModels",
                label="References", columns=3),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.SVCTypeAsset",
        title="SVCTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SVCTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ResistorTypeAsset" class:
#------------------------------------------------------------------------------

class ResistorTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic resistor that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ResistorAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.ResistorAssetModel"))

    Resistors = List(Instance("CIM.IEC61970.Wires.Resistor"))

    #--------------------------------------------------------------------------
    #  Begin "ResistorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "ResistorAssetModels", "Resistors",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.ResistorTypeAsset",
        title="ResistorTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ResistorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SeriesCompensatorTypeAsset" class:
#------------------------------------------------------------------------------

class SeriesCompensatorTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic series compensator that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ShuntCompensatorAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.SeriesCompensatorAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "SeriesCompensatorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "ShuntCompensatorAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.SeriesCompensatorTypeAsset",
        title="SeriesCompensatorTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SeriesCompensatorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "FaultIndicatorTypeAsset" class:
#------------------------------------------------------------------------------

class FaultIndicatorTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic fault indicator that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    FaultIndicatorAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.FaultIndicatorAssetModel"))

    FaultIndicators = List(Instance("CIM.IEC61970.Protection.FaultIndicator"))

    # Kind of reset mechanisim of this fault indicator.
    resetKind = FaultIndicatorResetKind(desc="Kind of reset mechanisim of this fault indicator.")

    #--------------------------------------------------------------------------
    #  Begin "FaultIndicatorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "resetKind",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "FaultIndicatorAssetModels", "FaultIndicators",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.FaultIndicatorTypeAsset",
        title="FaultIndicatorTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "FaultIndicatorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "VehicleTypeAsset" class:
#------------------------------------------------------------------------------

class VehicleTypeAsset(TypeAsset):
    """ Documentation for a generic vehicle that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    VehicleAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.VehicleAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "VehicleTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "VehicleAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.VehicleTypeAsset",
        title="VehicleTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "VehicleTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "BreakerTypeAsset" class:
#------------------------------------------------------------------------------

class BreakerTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic breaker asset that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    BreakerInfo = Instance("CIM.IEC61968.Informative.InfAssets.BreakerInfo",
        transient=True,
        opposite="BreakerTypeAsset",
        editor=InstanceEditor(name="_breakerinfos"))

    def _get_breakerinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.BreakerInfo" ]
        else:
            return []

    _breakerinfos = Property(fget=_get_breakerinfos)

    BreakerAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.BreakerAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "BreakerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "BreakerInfo", "BreakerAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.BreakerTypeAsset",
        title="BreakerTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "BreakerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PoleTypeAsset" class:
#------------------------------------------------------------------------------

class PoleTypeAsset(StructureTypeAsset):
    """ Documentation for a generic pole that may be used for various purposes such as work planning. A pole typically has a single Connection with 1,2 or 3 mounting points.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    PoleModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.PoleModel"))

    # Length of the pole (inclusive of any section of the pole that may be underground post-installation).
    length = Length(desc="Length of the pole (inclusive of any section of the pole that may be underground post-installation).")

    # Diameter of the pole.
    diameter = Length(desc="Diameter of the pole.")

    #--------------------------------------------------------------------------
    #  Begin "PoleTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "ratedVoltage", "length", "diameter",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "MountConnections", "PoleModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.PoleTypeAsset",
        title="PoleTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PoleTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DuctTypeAsset" class:
#------------------------------------------------------------------------------

class DuctTypeAsset(StructureTypeAsset):
    """ A Duct contains underground cables and is contained within a duct bank. The xCoord and yCoord attributes define its positioning within the DuctBank.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    CableAssets = List(Instance("CIM.IEC61968.Informative.InfAssets.CableAsset"))

    DuctBankTypeAsset = Instance("CIM.IEC61968.Informative.InfTypeAsset.DuctBankTypeAsset",
        transient=True,
        opposite="DuctTypeAssets",
        editor=InstanceEditor(name="_ductbanktypeassets"))

    def _get_ductbanktypeassets(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfTypeAsset.DuctBankTypeAsset" ]
        else:
            return []

    _ductbanktypeassets = Property(fget=_get_ductbanktypeassets)

    # X position of the duct within the duct bank.
    xCoord = Int(desc="X position of the duct within the duct bank.")

    # Y position of the duct within the duct bank.
    yCoord = Int(desc="Y position of the duct within the duct bank.")

    #--------------------------------------------------------------------------
    #  Begin "DuctTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "ratedVoltage", "xCoord", "yCoord",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "MountConnections", "CableAssets", "DuctBankTypeAsset",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.DuctTypeAsset",
        title="DuctTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DuctTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SurgeProtectorTypeAsset" class:
#------------------------------------------------------------------------------

class SurgeProtectorTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic surge arrestor that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    SurgeProtectors = List(Instance("CIM.IEC61970.Protection.SurgeProtector"))

    SurgeProtectorAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.SurgeProtectorAssetModel"))

    maximumEnergyAbsorption = Float

    maximumContinousOperatingVoltage = Voltage

    maximumCurrentRating = CurrentFlow

    nominalDesignVoltage = Voltage

    #--------------------------------------------------------------------------
    #  Begin "SurgeProtectorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "maximumEnergyAbsorption", "maximumContinousOperatingVoltage", "maximumCurrentRating", "nominalDesignVoltage",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "SurgeProtectors", "SurgeProtectorAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.SurgeProtectorTypeAsset",
        title="SurgeProtectorTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SurgeProtectorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ComFunctionTypeAsset" class:
#------------------------------------------------------------------------------

class ComFunctionTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic communication function that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ComFunctionAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.ComFunctionAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "ComFunctionTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "ComFunctionAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.ComFunctionTypeAsset",
        title="ComFunctionTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ComFunctionTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ProtectionEquipmentTypeAsset" class:
#------------------------------------------------------------------------------

class ProtectionEquipmentTypeAsset(ElectricalTypeAsset):
    """ Documentation for generic protection equiment that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    ProtectionEquipmentAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.ProtectionEquipmentAssetModel"))

    # Default phase trip setting for this type of relay, if applicable.
    defaultPhaseTrip = CurrentFlow(desc="Default phase trip setting for this type of relay, if applicable.")

    # Default ground trip setting for this type of relay, if applicable.
    defaultGroundTrip = CurrentFlow(desc="Default ground trip setting for this type of relay, if applicable.")

    #--------------------------------------------------------------------------
    #  Begin "ProtectionEquipmentTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "defaultPhaseTrip", "defaultGroundTrip",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "ProtectionEquipmentAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.ProtectionEquipmentTypeAsset",
        title="ProtectionEquipmentTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ProtectionEquipmentTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CabinetTypeAsset" class:
#------------------------------------------------------------------------------

class CabinetTypeAsset(StructureTypeAsset):
    """ Documentation for a generic cabinet that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    CabinetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.CabinetModel"))

    #--------------------------------------------------------------------------
    #  Begin "CabinetTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "ratedVoltage",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "MountConnections", "CabinetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.CabinetTypeAsset",
        title="CabinetTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CabinetTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "MeterTypeAsset" class:
#------------------------------------------------------------------------------

class MeterTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic meter that may be used for design purposes. Rather than being associated with CustomerMeter, it is associated with EnergyConsumer as it may be used for many applications, such as tie line metering, in addition to customer metering.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    MeterAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.MeterAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "MeterTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "MeterAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.MeterTypeAsset",
        title="MeterTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "MeterTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "GeneratorTypeAsset" class:
#------------------------------------------------------------------------------

class GeneratorTypeAsset(ElectricalTypeAsset):
    """ Documentation for generic generation equipment that may be used for various purposes such as work planning. It defines both the Real and Reactive power properties (modelled at the PSR level as a GeneratingUnit + SynchronousMachine)
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    GeneratorAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.GeneratorAssetModel"))

    # Direct-axis subtransient resistance
    rDirectSubtrans = Resistance(desc="Direct-axis subtransient resistance")

    # Direct-axis subtransient reactance
    xDirectSubtrans = Reactance(desc="Direct-axis subtransient reactance")

    # Quadrature-axis Transient resistance
    rQuadTrans = Resistance(desc="Quadrature-axis Transient resistance")

    # Quadrature-axis transient reactance.
    xQuadTrans = Reactance(desc="Quadrature-axis transient reactance.")

    # Quadrature-axis synchronous reactance
    xQuadSync = Reactance(desc="Quadrature-axis synchronous reactance")

    # Direct-axis synchronous reactance
    xDirectSync = Reactance(desc="Direct-axis synchronous reactance")

    # Quadrature-axis synchronous resistance
    rQuadSync = Resistance(desc="Quadrature-axis synchronous resistance")

    # Maximum reactive power limit.
    maxQ = ReactivePower(desc="Maximum reactive power limit.")

    # Direct-axis synchronous resistance
    rDirectSync = Resistance(desc="Direct-axis synchronous resistance")

    # Direct-axis Transient resistance
    rDirectTrans = Resistance(desc="Direct-axis Transient resistance")

    # Quadrature-axis subtransient resistance
    rQuadSubtrans = Resistance(desc="Quadrature-axis subtransient resistance")

    # Minimum reactive power generated.
    minQ = ReactivePower(desc="Minimum reactive power generated.")

    # Minimum real power generated.
    minP = ActivePower(desc="Minimum real power generated.")

    # Maximum real power limit.
    maxP = ActivePower(desc="Maximum real power limit.")

    # Direct-axis Transient reactance
    xDirectTrans = Reactance(desc="Direct-axis Transient reactance")

    # Quadrature-axis subtransient reactance
    xQuadSubtrans = Reactance(desc="Quadrature-axis subtransient reactance")

    #--------------------------------------------------------------------------
    #  Begin "GeneratorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "rDirectSubtrans", "xDirectSubtrans", "rQuadTrans", "xQuadTrans", "xQuadSync", "xDirectSync", "rQuadSync", "maxQ", "rDirectSync", "rDirectTrans", "rQuadSubtrans", "minQ", "minP", "maxP", "xDirectTrans", "xQuadSubtrans",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "GeneratorAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.GeneratorTypeAsset",
        title="GeneratorTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "GeneratorTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "SubstationTypeAsset" class:
#------------------------------------------------------------------------------

class SubstationTypeAsset(TypeAsset):
    """ Documentation for a type of substation that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    pass
    #--------------------------------------------------------------------------
    #  Begin "SubstationTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.SubstationTypeAsset",
        title="SubstationTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "SubstationTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "AssetFunctionTypeAsset" class:
#------------------------------------------------------------------------------

class AssetFunctionTypeAsset(TypeAsset):
    """ Documentation for a generic Asset Function that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    AssetFunctionAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.AssetFunctionAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "AssetFunctionTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "AssetFunctionAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.AssetFunctionTypeAsset",
        title="AssetFunctionTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "AssetFunctionTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "PotentialTransformerTypeAsset" class:
#------------------------------------------------------------------------------

class PotentialTransformerTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic Potential Transformer (PT) that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    nominalRatio = Instance("CIM.IEC61968.Informative.InfCommon.Ratio",
        transient=True,
        editor=InstanceEditor(name="_ratios"))

    def _get_ratios(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfCommon.Ratio" ]
        else:
            return []

    _ratios = Property(fget=_get_ratios)

    PotentialTransformers = List(Instance("CIM.IEC61970.Meas.PotentialTransformer"))

    PotentialTransformerAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.PotentialTransformerAssetModel"))

    PotentialTransformerInfo = Instance("CIM.IEC61968.Informative.InfAssets.PotentialTransformerInfo",
        transient=True,
        opposite="PotentialTransformerTypeAsset",
        editor=InstanceEditor(name="_potentialtransformerinfos"))

    def _get_potentialtransformerinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.PotentialTransformerInfo" ]
        else:
            return []

    _potentialtransformerinfos = Property(fget=_get_potentialtransformerinfos)

    ptClass = Str

    accuracyClass = Str

    #--------------------------------------------------------------------------
    #  Begin "PotentialTransformerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "ptClass", "accuracyClass",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "nominalRatio", "PotentialTransformers", "PotentialTransformerAssetModels", "PotentialTransformerInfo",
                label="References", columns=3),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.PotentialTransformerTypeAsset",
        title="PotentialTransformerTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "PotentialTransformerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "RecloserTypeAsset" class:
#------------------------------------------------------------------------------

class RecloserTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic recloser asset that may be used for design purposes.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    RecloserInfo = Instance("CIM.IEC61968.Informative.InfAssets.RecloserInfo",
        transient=True,
        opposite="RecloserTypeAsset",
        editor=InstanceEditor(name="_recloserinfos"))

    def _get_recloserinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.RecloserInfo" ]
        else:
            return []

    _recloserinfos = Property(fget=_get_recloserinfos)

    RecloserAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.RecloserAssetModel"))

    #--------------------------------------------------------------------------
    #  Begin "RecloserTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "RecloserInfo", "RecloserAssetModels",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.RecloserTypeAsset",
        title="RecloserTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "RecloserTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CurrentTransformerTypeAsset" class:
#------------------------------------------------------------------------------

class CurrentTransformerTypeAsset(ElectricalTypeAsset):
    """ Documentation for a generic Current Transformer (CT) that may be used for various purposes such as work planning.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    CurrentTransformerInfo = Instance("CIM.IEC61968.Informative.InfAssets.CurrentTransformerInfo",
        transient=True,
        opposite="CurrentTransformerTypeAsset",
        editor=InstanceEditor(name="_currenttransformerinfos"))

    def _get_currenttransformerinfos(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfAssets.CurrentTransformerInfo" ]
        else:
            return []

    _currenttransformerinfos = Property(fget=_get_currenttransformerinfos)

    CurrentTransformerAssetModels = List(Instance("CIM.IEC61968.Informative.InfAssetModels.CurrentTransformerAssetModel"))

    CurrentTransformers = List(Instance("CIM.IEC61970.Meas.CurrentTransformer"))

    # Nominal ratio between the primary and secondary current; i.e. 100:5
    nominalRatio = Instance("CIM.IEC61968.Informative.InfCommon.Ratio",
        desc="Nominal ratio between the primary and secondary current; i.e. 100:5",
        transient=True,
        editor=InstanceEditor(name="_ratios"))

    def _get_ratios(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfCommon.Ratio" ]
        else:
            return []

    _ratios = Property(fget=_get_ratios)

    # Maximum ratio between the primary and secondary current.
    maxRatio = Instance("CIM.IEC61968.Informative.InfCommon.Ratio",
        desc="Maximum ratio between the primary and secondary current.",
        transient=True,
        editor=InstanceEditor(name="_ratios"))

    def _get_ratios(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfCommon.Ratio" ]
        else:
            return []

    _ratios = Property(fget=_get_ratios)

    # eg. metering, protection, etc
    usage = Str(desc="eg. metering, protection, etc")

    # CT accuracy classification
    accuracyClass = Str(desc="CT accuracy classification")

    accuracyLimit = CurrentFlow

    # Maximum primary current where the CT still displays linear characteristicts.
    kneePointCurrent = CurrentFlow(desc="Maximum primary current where the CT still displays linear characteristicts.")

    # Power burden of the CT core
    coreBurden = ActivePower(desc="Power burden of the CT core")

    ctClass = Str

    # Number of cores.
    coreCount = Int(desc="Number of cores.")

    # Maximum voltage across the secondary terminals where the CT still displays linear characteristicts.
    kneePointVoltage = Voltage(desc="Maximum voltage across the secondary terminals where the CT still displays linear characteristicts.")

    #--------------------------------------------------------------------------
    #  Begin "CurrentTransformerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "usage", "accuracyClass", "accuracyLimit", "kneePointCurrent", "coreBurden", "ctClass", "coreCount", "kneePointVoltage",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "ElectricalInfos", "CurrentTransformerInfo", "CurrentTransformerAssetModels", "CurrentTransformers", "nominalRatio", "maxRatio",
                label="References", columns=3),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.CurrentTransformerTypeAsset",
        title="CurrentTransformerTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CurrentTransformerTypeAsset" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "DuctBankTypeAsset" class:
#------------------------------------------------------------------------------

class DuctBankTypeAsset(StructureTypeAsset):
    """ A DuctBank contains multiple Ducts. The DuctBank itself should have no connections, since these are defined by the individual ducts within it. However, it will have a ConstructionType and the material it is constructed from.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    DuctTypeAssets = List(Instance("CIM.IEC61968.Informative.InfTypeAsset.DuctTypeAsset"))

    DuctBanks = List(Instance("CIM.IEC61968.Informative.InfAssets.DuctBank"))

    #--------------------------------------------------------------------------
    #  Begin "DuctBankTypeAsset" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "subject", "revisionNumber", "category", "lastModifiedDateTime", "title", "createdDateTime", "quantity", "stockItem", "estimatedUnitCost", "ratedVoltage",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "ActivityRecords", "ErpOrganisationRoles", "ScheduledEvents", "FromDocumentRoles", "LocationRoles", "PowerSystemResourceRoles", "NetworkDataSets", "ErpPersonRoles", "ChangeItems", "Measurements", "docStatus", "ScheduleParameterInfos", "ElectronicAddress", "ToDocumentRoles", "status", "AssetRoles", "ChangeSets", "CUWorkEquipmentAsset", "CUAsset", "TypeAssetCatalogue", "AssetModels", "ErpInventoryIssues", "ErpReqLineItems", "ErpBomItemDatas", "MountConnections", "DuctTypeAssets", "DuctBanks",
                label="References", columns=2),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfTypeAsset.DuctBankTypeAsset",
        title="DuctBankTypeAsset",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "DuctBankTypeAsset" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CPSM.IEC61970.Core import ConnectivityNodeContainer
from CPSM.IEC61970.Core import ConductingEquipment
from CPSM.IEC61970.Domain import Susceptance
from CPSM.IEC61970.Domain import Conductance
from CPSM.IEC61970.Domain import Reactance
from CPSM.IEC61970.Domain import Resistance



from enthought.traits.api import Instance, List, Property
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------


#------------------------------------------------------------------------------
#  "EquivalentNetwork" class:
#------------------------------------------------------------------------------

class EquivalentNetwork(ConnectivityNodeContainer):
    """ A class that represents an external meshed network that has been reduced to an electrically equivalent model. The ConnectivityNodes contained in the equivalent are intended to reflect internal nodes of the equivalent. The boundary Connectivity nodes where the equivalent connects outside itself are NOT contained by the equivalent.A class that represents an external meshed network that has been reduced to an electrically equivalent model. The ConnectivityNodes contained in the equivalent are intended to reflect internal nodes of the equivalent. The boundary Connectivity nodes where the equivalent connects outside itself are NOT contained by the equivalent.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The associated reduced equivalents.The associated reduced equivalents.
    EquivalentEquipments = List(Instance("CPSM.IEC61970.Equivalents.EquivalentEquipment"),
        desc="The associated reduced equivalents.The associated reduced equivalents.")

    #--------------------------------------------------------------------------
    #  Begin "EquivalentNetwork" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "ConnectivityNodes", "EquivalentEquipments",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.Equivalents.EquivalentNetwork",
        title="EquivalentNetwork",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EquivalentNetwork" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EquivalentEquipment" class:
#------------------------------------------------------------------------------

class EquivalentEquipment(ConductingEquipment):
    """ The class represents equivalent objects that are the result of a network reduction. The class is the base for equivalent objects of diferent types.The class represents equivalent objects that are the result of a network reduction. The class is the base for equivalent objects of diferent types.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The equivalent where the reduced model belongs.The equivalent where the reduced model belongs.
    EquivalentNetwork = Instance("CPSM.IEC61970.Equivalents.EquivalentNetwork", allow_none=False,
        desc="The equivalent where the reduced model belongs.The equivalent where the reduced model belongs.",
        transient=True,
        opposite="EquivalentEquipments",
        editor=InstanceEditor(name="_equivalentnetworks"))

    def _get_equivalentnetworks(self):
        """ Property getter.
        """
        if self.Model is not None:
            return [e for e in self.Model.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CPSM.IEC61970.Equivalents.EquivalentNetwork" ]
        else:
            return []

    _equivalentnetworks = Property(fget=_get_equivalentnetworks)

    #--------------------------------------------------------------------------
    #  Begin "EquivalentEquipment" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "EquivalentNetwork",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.Equivalents.EquivalentEquipment",
        title="EquivalentEquipment",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EquivalentEquipment" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EquivalentShunt" class:
#------------------------------------------------------------------------------

class EquivalentShunt(EquivalentEquipment):
    """ The class represents equivalent shunts.The class represents equivalent shunts.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Positive sequence shunt susceptance.Positive sequence shunt susceptance.
    b = Susceptance(desc="Positive sequence shunt susceptance.Positive sequence shunt susceptance.")

    # Positive sequence shunt conductance.Positive sequence shunt conductance.
    g = Conductance(desc="Positive sequence shunt conductance.Positive sequence shunt conductance.")

    #--------------------------------------------------------------------------
    #  Begin "EquivalentShunt" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "b", "g",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "EquivalentNetwork",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.Equivalents.EquivalentShunt",
        title="EquivalentShunt",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EquivalentShunt" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "EquivalentBranch" class:
#------------------------------------------------------------------------------

class EquivalentBranch(EquivalentEquipment):
    """ The class represents equivalent branches.The class represents equivalent branches.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Positive sequence series reactance of the reduced branch.Positive sequence series reactance of the reduced branch.
    x = Reactance(desc="Positive sequence series reactance of the reduced branch.Positive sequence series reactance of the reduced branch.")

    # Positive sequence series resistance of the reduced branch.Positive sequence series resistance of the reduced branch.
    r = Resistance(desc="Positive sequence series resistance of the reduced branch.Positive sequence series resistance of the reduced branch.")

    #--------------------------------------------------------------------------
    #  Begin "EquivalentBranch" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "pathName", "description", "aliasName", "name", "x", "r",
                label="Attributes"),
            VGroup("Model", "Contains_Measurements", "MemberOf_EquipmentContainer", "OperationalLimitSet", "BaseVoltage", "Terminals", "EquivalentNetwork",
                label="References"),
            dock="tab"),
        id="CPSM.IEC61970.Equivalents.EquivalentBranch",
        title="EquivalentBranch",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "EquivalentBranch" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


from cpsm.iec61970.core import ConnectivityNodeContainer
from cpsm.iec61970.core import ConductingEquipment

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim"

ns_uri = "http://iec.ch/TC57/2008/CIM-schema-cim13#Package_Equivalents"

class EquivalentNetwork(ConnectivityNodeContainer):
    """ A class that represents an external meshed network that has been reduced to an electrically equivalent model. The ConnectivityNodes contained in the equivalent are intended to reflect internal nodes of the equivalent. The boundary Connectivity nodes where the equivalent connects outside itself are NOT contained by the equivalent.A class that represents an external meshed network that has been reduced to an electrically equivalent model. The ConnectivityNodes contained in the equivalent are intended to reflect internal nodes of the equivalent. The boundary Connectivity nodes where the equivalent connects outside itself are NOT contained by the equivalent.
    """
    # <<< equivalent_network
    # @generated
    def __init__(self, equivalent_equipments=None, **kw_args):
        """ Initialises a new 'EquivalentNetwork' instance.
        """

        self._equivalent_equipments = []
        if equivalent_equipments is not None:
            self.equivalent_equipments = equivalent_equipments
        else:
            self.equivalent_equipments = []


        super(EquivalentNetwork, self).__init__(**kw_args)
    # >>> equivalent_network

    # <<< equivalent_equipments
    # @generated
    def get_equivalent_equipments(self):
        """ The associated reduced equivalents.The associated reduced equivalents.
        """
        return self._equivalent_equipments

    def set_equivalent_equipments(self, value):
        for x in self._equivalent_equipments:
            x._equivalent_network = None
        for y in value:
            y._equivalent_network = self
        self._equivalent_equipments = value

    equivalent_equipments = property(get_equivalent_equipments, set_equivalent_equipments)

    def add_equivalent_equipments(self, *equivalent_equipments):
        for obj in equivalent_equipments:
            obj._equivalent_network = self
            self._equivalent_equipments.append(obj)

    def remove_equivalent_equipments(self, *equivalent_equipments):
        for obj in equivalent_equipments:
            obj._equivalent_network = None
            self._equivalent_equipments.remove(obj)
    # >>> equivalent_equipments


    def __str__(self):
        """ Returns a string representation of the EquivalentNetwork.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< equivalent_network.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the EquivalentNetwork.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "EquivalentNetwork", self.uri)
        if format:
            indent += ' ' * depth

        for obj in self.equivalent_equipments:
            s += '%s<%s:EquivalentNetwork.equivalent_equipments rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        for obj in self.contains_measurements:
            s += '%s<%s:PowerSystemResource.contains_measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.connectivity_nodes:
            s += '%s<%s:ConnectivityNodeContainer.connectivity_nodes rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "EquivalentNetwork")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> equivalent_network.serialize


class EquivalentEquipment(ConductingEquipment):
    """ The class represents equivalent objects that are the result of a network reduction. The class is the base for equivalent objects of diferent types.The class represents equivalent objects that are the result of a network reduction. The class is the base for equivalent objects of diferent types.
    """
    # <<< equivalent_equipment
    # @generated
    def __init__(self, equivalent_network=None, **kw_args):
        """ Initialises a new 'EquivalentEquipment' instance.
        """

        self._equivalent_network = None
        self.equivalent_network = equivalent_network


        super(EquivalentEquipment, self).__init__(**kw_args)
    # >>> equivalent_equipment

    # <<< equivalent_network
    # @generated
    def get_equivalent_network(self):
        """ The equivalent where the reduced model belongs.The equivalent where the reduced model belongs.
        """
        return self._equivalent_network

    def set_equivalent_network(self, value):
        if self._equivalent_network is not None:
            filtered = [x for x in self.equivalent_network.equivalent_equipments if x != self]
            self._equivalent_network._equivalent_equipments = filtered

        self._equivalent_network = value
        if self._equivalent_network is not None:
            self._equivalent_network._equivalent_equipments.append(self)

    equivalent_network = property(get_equivalent_network, set_equivalent_network)
    # >>> equivalent_network


    def __str__(self):
        """ Returns a string representation of the EquivalentEquipment.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< equivalent_equipment.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the EquivalentEquipment.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "EquivalentEquipment", self.uri)
        if format:
            indent += ' ' * depth

        if self.equivalent_network is not None:
            s += '%s<%s:EquivalentEquipment.equivalent_network rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.equivalent_network.uri)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        for obj in self.contains_measurements:
            s += '%s<%s:PowerSystemResource.contains_measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.member_of_equipment_container is not None:
            s += '%s<%s:Equipment.member_of_equipment_container rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.member_of_equipment_container.uri)
        for obj in self.operational_limit_set:
            s += '%s<%s:Equipment.operational_limit_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.base_voltage is not None:
            s += '%s<%s:ConductingEquipment.base_voltage rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.base_voltage.uri)
        for obj in self.terminals:
            s += '%s<%s:ConductingEquipment.terminals rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "EquivalentEquipment")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> equivalent_equipment.serialize


class EquivalentShunt(EquivalentEquipment):
    """ The class represents equivalent shunts.The class represents equivalent shunts.
    """
    # <<< equivalent_shunt
    # @generated
    def __init__(self, b=0.0, g=0.0, **kw_args):
        """ Initialises a new 'EquivalentShunt' instance.
        """
        # Positive sequence shunt susceptance.Positive sequence shunt susceptance. 
        self.b = b

        # Positive sequence shunt conductance.Positive sequence shunt conductance. 
        self.g = g



        super(EquivalentShunt, self).__init__(**kw_args)
    # >>> equivalent_shunt


    def __str__(self):
        """ Returns a string representation of the EquivalentShunt.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< equivalent_shunt.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the EquivalentShunt.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "EquivalentShunt", self.uri)
        if format:
            indent += ' ' * depth

        s += '%s<%s:EquivalentShunt.b>%s</%s:EquivalentShunt.b>' % \
            (indent, ns_prefix, self.b, ns_prefix)
        s += '%s<%s:EquivalentShunt.g>%s</%s:EquivalentShunt.g>' % \
            (indent, ns_prefix, self.g, ns_prefix)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        for obj in self.contains_measurements:
            s += '%s<%s:PowerSystemResource.contains_measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.member_of_equipment_container is not None:
            s += '%s<%s:Equipment.member_of_equipment_container rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.member_of_equipment_container.uri)
        for obj in self.operational_limit_set:
            s += '%s<%s:Equipment.operational_limit_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.base_voltage is not None:
            s += '%s<%s:ConductingEquipment.base_voltage rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.base_voltage.uri)
        for obj in self.terminals:
            s += '%s<%s:ConductingEquipment.terminals rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.equivalent_network is not None:
            s += '%s<%s:EquivalentEquipment.equivalent_network rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.equivalent_network.uri)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "EquivalentShunt")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> equivalent_shunt.serialize


class EquivalentBranch(EquivalentEquipment):
    """ The class represents equivalent branches.The class represents equivalent branches.
    """
    # <<< equivalent_branch
    # @generated
    def __init__(self, x=0.0, r=0.0, **kw_args):
        """ Initialises a new 'EquivalentBranch' instance.
        """
        # Positive sequence series reactance of the reduced branch.Positive sequence series reactance of the reduced branch. 
        self.x = x

        # Positive sequence series resistance of the reduced branch.Positive sequence series resistance of the reduced branch. 
        self.r = r



        super(EquivalentBranch, self).__init__(**kw_args)
    # >>> equivalent_branch


    def __str__(self):
        """ Returns a string representation of the EquivalentBranch.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< equivalent_branch.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the EquivalentBranch.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "EquivalentBranch", self.uri)
        if format:
            indent += ' ' * depth

        s += '%s<%s:EquivalentBranch.x>%s</%s:EquivalentBranch.x>' % \
            (indent, ns_prefix, self.x, ns_prefix)
        s += '%s<%s:EquivalentBranch.r>%s</%s:EquivalentBranch.r>' % \
            (indent, ns_prefix, self.r, ns_prefix)
        if self.model is not None:
            s += '%s<%s:Element.model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.model.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        for obj in self.contains_measurements:
            s += '%s<%s:PowerSystemResource.contains_measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.member_of_equipment_container is not None:
            s += '%s<%s:Equipment.member_of_equipment_container rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.member_of_equipment_container.uri)
        for obj in self.operational_limit_set:
            s += '%s<%s:Equipment.operational_limit_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.base_voltage is not None:
            s += '%s<%s:ConductingEquipment.base_voltage rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.base_voltage.uri)
        for obj in self.terminals:
            s += '%s<%s:ConductingEquipment.terminals rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.equivalent_network is not None:
            s += '%s<%s:EquivalentEquipment.equivalent_network rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.equivalent_network.uri)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "EquivalentBranch")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> equivalent_branch.serialize


# <<< equivalents
# @generated
# >>> equivalents
